//---------------------------------------------------------------------------
//
//                     XMM Science Analysis System (SAS)
//                    (c) 1997-2010 European Space Agency
//
//---------------------------------------------------------------------------
//
//	RegionFilter.h: --- region filter related classes
//
//	DESCRIPTION:
//		Declaration of region filter related classes
//
//	CHANGE HISTORY:
//		UL, 2001-08-07: 1.0.0: - first version
//
//	AUTHOR:
//		U. Lammers (UL), Tue Aug  7 10:54:08 MET DST 2001
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/selectlib/src/selcore/RegionFilter.h,v 1.3 2001/08/15 16:44:07 ulammers Exp $
///

#ifndef POLYGON_H
#define POLYGON_H

#include "Selector.h"
#include "TriangulatablePolygon.h"

#include <vector>

class NodePoint {
public:
	NodePoint() : _x(0), _y(0) {}
	NodePoint(OpNode x, OpNode y) : _x(x), _y(y) {}
	inline OpNode xNode() { return _x; }
	inline OpNode yNode() { return _y; }
	~NodePoint() {}

private:
	OpNode _x, _y;
};

class NodePointList : public OperatorNode, public std::vector<NodePoint> {
public:
	NodePointList() {}
	void addPoint(const OpNode x, const OpNode y);
	~NodePointList() {}
};

#define BEGINCLASS(a)					\
class a : public OperatorNode {			\
public:									\
	a();								\
	~a();

#define ENDCLASS };

#define OPCLASS3(a)		BEGINCLASS(a) a(const OpNode, const OpNode,			\
										const OpNode);						\
						ENDCLASS

//
//	operator classes
//
class TriangulatablePolygonFilter : public OperatorNode,
									public TriangulatablePolygon {
public:
	TriangulatablePolygonFilter(const std::vector<double> &xCoords,
								const std::vector<double> &yCoords,
								const OpNode testX,
								const OpNode testY,
								bool testSegments = false);

	~TriangulatablePolygonFilter();
};

OPCLASS3(VariablePolygonFilter)

namespace PolygonScope {
	//	virtual constructor: decide on constness of point list whether
	//	to construct a standard polygon or a TriangulatablePolygon
	OpNode newPolyFilter(const OpNode,	//	point list
					     const OpNode,	//	text point x
						 const OpNode,	//	test point y
						 bool testSegments = false);
}

#undef BEGINCLASS
#undef ENDCLASS
#undef OPCLASS3

#endif
