//----------------------------------------------------------------------------
//
//                     XMM Science Analysis System (SAS)
//                    (c) 1997-2010 European Space Agency
//
//---------------------------------------------------------------------------
//
//	OGIPgti.h: --- read/write OGIP GTI files
//
//	DESCRIPTION:
//		Instances of class OGIPgtiTable represent GTI filters according
//		to OGIP conventions
//
//	CHANGE HISTORY:
//		UL, 1998-03-25: 1.0.0: - first version
//		UL, 2002-01-29: 1.1.0: - added OutOGIPgtiTable::mergeTimingAttributes()
//		UL, 2002-06-10: 1.2.0: - added OGIPgtiTable::table()
//
//	AUTHOR:
//		U. Lammers (UL), Wed Mar 25 16:52:23 MET 1998
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/selectlib/src/utils/OGIPgti.h,v 1.5 2002/06/10 13:12:55 ulammers Exp $
//

#ifndef OGIPGTI_H
#define OGIPGTI_H

#include <string>
#include "Gti.h"
#include "Dal.h"

const std::string GTISTARTCOLNAME	= "START",
				  GTISTOPCOLNAME	= "STOP";

class OGIPgtiTable {
public:
	OGIPgtiTable() {}
	OGIPgtiTable(const std::string &setName, bool closeSet = true);
	virtual ~OGIPgtiTable() {}

	std::string setName() const { return _setName; }
	std::string startColName() const { return GTISTARTCOLNAME; }
	std::string stopColName() const { return GTISTOPCOLNAME; }

	Table *table() const { return _table; }

protected:
	//	data members
	DataSet		*_set;
	Table		*_table;
	std::string	_setName;
	bool		_closeSet;	//	close set in destructor?
}; // OGIPgtiTable


class InOGIPgtiTable : public virtual OGIPgtiTable {
public:
	InOGIPgtiTable() {}

	//	construct from data set name - entails opening of it
	InOGIPgtiTable(const std::string &setName);

	//	construct from open data set
	InOGIPgtiTable(DataSet *dataSet);

	//	construct from table
	InOGIPgtiTable(Table *table);

	//	destructor
	~InOGIPgtiTable();	

	friend InOGIPgtiTable &operator>>(InOGIPgtiTable &, GtiList &);
}; // InOGIPgtiTable


class OutOGIPgtiTable : public virtual OGIPgtiTable {
public:
	OutOGIPgtiTable() {}

		//	construct from set name - entails opening a new data set
	OutOGIPgtiTable(const std::string &setName);

	//	construct from open set
	OutOGIPgtiTable(DataSet *dataSet);

	//	construct from table
	OutOGIPgtiTable(Table *table);

	//	destructor
	~OutOGIPgtiTable();

	friend OutOGIPgtiTable &operator<<(OutOGIPgtiTable &, const GtiList &);

private:
	void setAttribute(const std::string &, const std::string &,
					  const std::string &);
	void setFromSet(DataSet *dataSet, const std::string &tableName = "");
	void setFromTable(Table *table);
	void mergeTimingAttributes(const GtiList &);
}; // OutOGIPgtiTable

InOGIPgtiTable &operator>>(InOGIPgtiTable &, GtiList &);
OutOGIPgtiTable &operator<<(OutOGIPgtiTable &, const GtiList &);

#endif
